#ifndef AGDDCOMPOSHH
#define AGDDCOMPOSHH

namespace agdd {
  class Compos;
}

#include <vector>
#include "agddSection.hh"
#include "agddVolume.hh"
#include "agddPos.hh"
#include "agddHasTag.hh"
#include "agddPosContainer.hh"
#include "agddHelpStan.hh"
#include "ExpatInterface/Element.h"

namespace agdd {

  class Compos : public Volume, public HasTag , public PosContainer {
  private:
    //bool _writeProtect;
    vector<Pos *> _posVec; // Pointed to for reference.  
                           // May or may not be our responsibility.
  public:
    Compos() :
      Volume(),
      HasTag("composition"),
      PosContainer(),
      //_writeProtect(false),
      _posVec() {
      };
    ~Compos() {
    };

    xml::Element * posElementsGoHere() {
      assert(thisVolumeIsAssignedToASection());
      return theElement();
    };

    Section * defaultSection() {
      return getSection();
    };

    void putMyselfIn(Section * sec) {
      assert(sec);
      /*
	// protect against infinite looping through circular chain:
	if (_writeProtect) {
	cerr << "Error: Circular set of compositions." << endl;
	};
	assert(!_writeProtect);
      */
      // If this composition is already being assigned 
      // to a section, we need not go further.
      if (thisVolumeIsNotAssignedToASection()) {
	//_writeProtect=true;
	cout << "Creating a " << flush << getTag()
	     << " named " << flush << getName() 
	     << " ... " << endl;
	makeTheElement();
	// The following line worked fine where it was, but to try
        // to make up for defeciencies in some AGDD readers which
        // expect IDREFS to appear *before* they are referenced,
        // I have placed it after the _posVec[] loop below. 

	// putThisVolumeElementIntoThisSection(theElement(), sec);

	// And the next line is completely new for stan
	if (helpStan) {
	  setSectionForThisVolume(sec);
	} else {
	  putThisVolumeElementIntoThisSection(theElement(), sec);
	};

	doVolumeThingsToThisElement(theElement());
	// Now we need to put in the poses.
	{
	  for (unsigned int i=0; i<_posVec.size(); i++) {
	    _posVec[i]->putMeInThisPosContainer(this);
	  }
	} 
	
	if (helpStan) {
	  putTheFollowingVolumeElementIntoItsSection(theElement());
	};

	cout << "done." << endl;
        //_writeProtect=false;
      }      
    };
  public:
    void addPos(Pos * pos) {
      _posVec.push_back(pos);
    };
  };

} // end of namespace

#endif
